/** \file hcms-297x.c -    HCMS-29xx
 * \brief       HCMS-29xx
 * \par
 *     ,    
 *   .
 * \par \author ARV \par
 * \note     
 * \n :
 * \n \date	27 . 2016 .
 * \par
 * \version 0.01.	\par
 * Copyright 2015  ARV. All rights reserved.
 * \par
 *   :\n
 * 	-# WinAVR-20100110    
 *
 */

#include <avr/io.h>
#include <avr_helper.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <string.h>

#include "hcms_297x.h"

//        GCC,  __flash
#if (__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ > 3))
#define p_get(x)	(x)
#else
#define __flash		PROGMEM
#define p_get(x)	pgm_read_byte(&x)
#endif

//       ,   
#include "font5x7.h"

//  
char hcms_screen[SCR_SIZE];

#if defined(USE_CURSOR)
static uint8_t cursor_pos = NO_CURSOR;
#endif

//  
static ctrl_reg0_t def_ctrl_r0;
static ctrl_reg1_t def_ctrl_r1;

///       
#define hcms_ctrl_mode()	PORT(PORT_RS) |= _BV(PIN_RS)
///     
#define hcms_data_mode()	PORT(PORT_RS) &= ~_BV(PIN_RS)
///    
#define hcms_enable()		PORT(PORT_CE) &= ~_BV(PIN_CE)
///    
#define hcms_disable()		PORT(PORT_CE) |= _BV(PIN_CE)
#if !defined(USE_SPI)
///       CLK
#define hcms_clk_hi()		PORT(PORT_CLK) |= _BV(PIN_CLK)
///       CLK
#define hcms_clk_lo()		PORT(PORT_CLK) &= ~_BV(PIN_CLK)
///       
#define hcms_di_hi()		PORT(PORT_DI) |= _BV(PIN_DI)
///       
#define hcms_di_lo()		PORT(PORT_DI) &= ~_BV(PIN_DI)
#else
///       CLK
#define hcms_clk_hi()
///       CLK
#define hcms_clk_lo()
///       
#define hcms_di_hi()
///       
#define hcms_di_lo()
#endif

/**  
 *    -  
 */
static void hcms_reset(void){
	PORT(PORT_RST) &= ~_BV(PIN_RST);
	PORT(PORT_RST) |= _BV(PIN_RST);
}

/**    
 *         #USE_SPI
 *  ,  
 * \note     5   
 * @param c
 */
#if !defined(USE_SPI)
static void hcms_put_byte(uint8_t c){
	for(uint8_t mask = 0x80; mask; mask >>= 1){
		hcms_clk_lo();
		if(c & mask)	hcms_di_hi();
		else			hcms_di_lo();
		hcms_clk_hi();
	}
}
#else
static void hcms_put_byte(uint8_t c){
	SPDR = c;
	while(!(SPSR & _BV(SPIF)));
}
#endif

/**   
 *
 * @param data       
 */
static void hcms_ctrl_register(uint8_t data){
	hcms_ctrl_mode();
	hcms_enable();
	hcms_put_byte(data);
	hcms_clk_lo();
	hcms_disable();
}

//  "" 
void hcms_raw_pixels(uint8_t *buf, buf_size_t sz){
	hcms_data_mode();
	hcms_enable();
	for(buf_size_t i=0; i < sz; i++){
		hcms_put_byte(buf[i]);
	}
	hcms_clk_lo();
	hcms_disable();
}

static void rpad_strcpy(char *d, char *s){
	uint8_t l = strlen(s);
	for(uint8_t i=0; i<SCR_SIZE; i++)
		if(i >= l)
			d[i] = ' ';
		else
			d[i] = s[i];
}

//  
void hcms_puts(char *s){
	rpad_strcpy(hcms_screen, s);
	hcms_update_scr();
}

//    flash
void hcms_puts_P(const char *s){
	char tmp[SCR_SIZE];
	strncpy_P(tmp, s, SCR_SIZE);
	hcms_puts(tmp);
}

/**       
 *
 * @param c 
 * @return      
 */
static uint16_t get_offset_char(char c){
	if(c){
		//     
		if((c < FONT_FIRST_CHAR) || (c > FONT_LAST_CHAR)) c = FONT_UNKNOWN_CHAR;
	} else {
		//   '\0' -    
		c = ' ';
	}

	return (c - FONT_FIRST_CHAR) * FONT_COLS;
}

#if defined(USE_CURSOR)
static uint8_t inverce;
#endif

static void hcms_put_char(char c){
	uint16_t idx; //     

	//  
	idx  = get_offset_char(c);
	//  "" 
	for(uint8_t i = 0; i < FONT_COLS; i++){
		//     flash    #p_get
		hcms_put_byte(p_get(font5x7[idx++])
#if defined(USE_CURSOR)
			//   ,      
			^ (inverce ? 0xFF : 0)
#endif
		);
	}
}

//  
void hcms_clrscr(void){
#if defined(USE_CURSOR)
	cursor_pos = NO_CURSOR;
#endif
	hcms_puts_P(PSTR(""));
}

//     
void hcms_update_scr(void){
	hcms_data_mode();
	hcms_enable();

	for(uint8_t pos = 0; pos < SCR_SIZE; pos++){
#if defined(USE_CURSOR)
		//   ,      
		inverce = (pos == cursor_pos);
#endif
		hcms_put_char(hcms_screen[pos]);
	}
	hcms_clk_lo();
	hcms_disable();
}

#if defined(USE_CURSOR)
//   
void hcms_cursor_pos(uint8_t pos){
	cursor_pos = pos;
	hcms_update_scr());
}
#endif

//  
void hcms_bright(pwm_brightness_t br){
	def_ctrl_r0.brightness = br;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

// - 
void hcms_on(uint8_t on){
	def_ctrl_r0.sleep_mode = !on;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

//   
void hcms_peak_current(peak_current_t pc){
	def_ctrl_r0.peak_current = pc;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

//  
INIT(7){
#if defined(ONE_PORT)
	DDR(ONE_PORT) |= _BV(PIN_CE) | _BV(PIN_RS) | _BV(PIN_RST);
	PORT(ONE_PORT) |= _BV(PIN_CE) | _BV(PIN_RS) | _BV(PIN_RST);
#else
	DDR(PORT_CE) |= _BV(PIN_CE);
	PORT(PORT_CE) |= _BV(PIN_CE);
	DDR(PORT_RS) |= _BV(PIN_RS);
	PORT(PORT_RS) |= _BV(PIN_RS);
	DDR(PORT_RST) |= _BV(PIN_RST);
#endif
	hcms_reset();

#if defined(USE_SPI)
	//   SPI
	SPCR = _BV(SPE) | _BV(MSTR) | SPI_CLK_DIV_4;
	SPSR = _BV(SPI2X);
#if defined(__AVR_ATmega32__) || defined(__AVR_ATmega32A__) || defined(__AVR_ATmega16__)
	DDRB |= _BV(PB7) | _BV(PB5) | _BV(PB4); // SS, MOSI, SCK
	DDRB &= ~ _BV(PB6); // MISO
#elif defined(__AVR_ATmega8__)
	DDRB |= _BV(PB2) | _BV(PB5) | _BV(PB3); // SS, MOSI, SCK
	DDRB &= ~_BV(PB4); // MISO
#else
#	error CONFIGURE MANUALLY SS PIN AS OUTPUT, and so MOSI and CLK.
#endif
#else
#if defined(ONE_PORT)
	DDR(ONE_PORT) |= _BV(PIN_CLK) | _BV(PIN_DI);
#else
	DDR(PORT_CLK) |= _BV(PIN_CLK);
	DDR(PORT_DI) |= _BV(PIN_DI);
#endif
	PORT(PORT_CLK) &= ~_BV(PIN_CLK);
#endif

	//      

	def_ctrl_r0.brightness = DEFAULT_BRIGHTNESS;
	def_ctrl_r0.peak_current = DEFAULT_PEAK_CURRENT;
	def_ctrl_r0.sleep_mode = 1;
	def_ctrl_r0.reg = 0;

	def_ctrl_r1.data_out_ctrl = 1;
	def_ctrl_r1.edo_prescaler = 0;
	def_ctrl_r1.reserved = 0;
	def_ctrl_r1.reg = 1;

	hcms_clrscr();
	//    1    
	//   ,     
	//       .   
	//       1-   ,
	//    .
	for(uint8_t i=0; i < CHIP_CNT+1; i++)
		hcms_ctrl_register(def_ctrl_r1.byte);
	hcms_ctrl_register(def_ctrl_r0.byte);
}

/*****************************************************************************
 *   "" ,   
 * ,       .
 *****************************************************************************/
#include <util/delay.h>
#include <stdio.h>

/**      
 *       ,   
 *   
 * @param c0  
 * @param c1  
 * @param shift   
 */
static void shiftdn_two_char(uint8_t c0, uint8_t c1, uint8_t shift){
	uint8_t b0, b1;
	for(uint8_t i=0; i < FONT_COLS; i++){
		b0 = p_get(font5x7[get_offset_char(c0) + i]);
		b1 = p_get(font5x7[get_offset_char(c1) + i]);
		for(uint8_t sh = shift; sh; sh--){
			b0 = (b0 << 1) | (b1 & 0x80 ? 1 : 0);
			b1 <<= 1;
		}
		hcms_put_byte(b0);
	}
}

/**      
 *       ,   
 *   
 * @param c0  
 * @param c1  
 * @param shift   
 */
static void shiftup_two_char(uint8_t c0, uint8_t c1, uint8_t shift){
	uint8_t b0, b1;
	for(uint8_t i=0; i < FONT_COLS; i++){
		b0 = p_get(font5x7[get_offset_char(c0) + i]);
		b1 = p_get(font5x7[get_offset_char(c1) + i]);
		for(uint8_t sh = shift; sh; sh--){
			b0 = (b0 >> 1) | (b1 & 1 ? 0x80 : 0);
			b1 >>= 1;
		}
		hcms_put_byte(b0);
	}
}

/**     " "
 *  
 * @param buf   
 * @param sz   
 * @return 0,       , 1 -    
 *   
 */
static uint8_t is_clear(uint8_t *buf, uint8_t sz){
	uint8_t result = 0;
	for(;sz;sz--)
		result |= *buf++;
	return result == 0;
}

///     
static char ns[SCR_SIZE];

/**          
 *      
 *  ()   ()
 * @param up : 1 - , -1 - 
 * @param pos  
 * @param shift
 */
static void hcms_rollover_chars(int8_t up, uint8_t pos, uint8_t shift){
	if(hcms_screen[pos] != ns[pos]){
		if(up < 0)	shiftdn_two_char(hcms_screen[pos], ns[pos], shift);
		else		shiftup_two_char(hcms_screen[pos], ns[pos], shift);
	} else {
		hcms_put_char(ns[pos]);
	}
}

//     /
void hcms_rollower_puts(char *s, int8_t up){
	//     
	rpad_strcpy(ns, s);
	//      
	for(uint8_t sh=1; sh <=8; sh++){
		hcms_data_mode();
		hcms_enable();
		//   
		for(uint8_t pos=0; pos < SCR_SIZE; pos++){
			//         
			hcms_rollover_chars(up, pos, sh);
		}
		hcms_clk_lo();
		hcms_disable();
		//   
		_delay_ms(ANIMATE_DELAY_MS);
	}
	//      
	memcpy(hcms_screen, ns, SCR_SIZE);
}

//      /
void hcms_smooth_rollower_puts(char *s, int8_t up){
	//   
	rpad_strcpy(ns, s);

	static uint8_t sh[SCR_SIZE];	//   
	memset(sh, 0, SCR_SIZE);		//   ,
	sh[0] = 1;						//    

	//   ,        
	//   
	while(!is_clear(sh, SCR_SIZE)){
		hcms_data_mode();
		hcms_enable();

		//          
		for(uint8_t pos=0; pos < SCR_SIZE; pos++){
			//        -  
			hcms_rollover_chars(up, pos, sh[pos]);
		}
		//   ,    
		//     1 
		memmove(sh+1, sh, SCR_SIZE-1);
		//      
		if(sh[0]){
			//    1,   ,   8
			if(++(sh[0]) > 8){
				//  ,     9 - ,
				// ..    
				sh[0] = 0;
			}
		}

		//   
		for(uint8_t i=0; i < SCR_SIZE; i++){
			//       ,    
			// 
			if(sh[i] == 0) hcms_screen[i] = ns[i];
			else break;
		}

		hcms_clk_lo();
		hcms_disable();
		//   
		_delay_ms(ANIMATE_DELAY_MS);
	}
}
