/** \file hcms-297x.h -    HCMS-29xx
 * \brief , , ,     .
 * \par
 *   :
 * -#   5  
 * -#     SPI
 * -#     - 4, 8  16 
 * -#   
 * \par
 * \par \author ARV \par
 * \note     
 * \n : . datasheet
 * \n \date	27 . 2016 .
 * \par
 * \version 0.01	\par
 * Copyright 2015  ARV. All rights reserved.
 * \par
 *   :\n
 * 	-# WinAVR-20100110    
 *
 */

#ifndef HCMS_297X_H_
#define HCMS_297X_H_

#include <avr/io.h>
#include <stdint.h>

///         SPI,
///     (     5 )
#define USE_SPI

///     -   .
/// ,    -   
//#define USE_CURSOR

///   
#define		CHIP_CNT		2
///    1 
#define		CHIP_POS		8
///     (  )
#define 	FONT_COLS		5
///  ,   
#define		SCR_SIZE		(CHIP_CNT*CHIP_POS)
///     
#define		CHIP_RAW_SIZE	(CHIP_CNT*CHIP_POS*FONT_COLS)

///      1  -   ,
///    !
#define		ONE_PORT		D

//         
//   -     , ..   0  7
///   
#define		PORT_CE			ONE_PORT
///   
#define		PIN_CE			1

///   
#define		PORT_RS			ONE_PORT
///   
#define		PIN_RS			3

///  
#define		PORT_RST		ONE_PORT
///  
#define		PIN_RST			0

#if !defined(USE_SPI)
//     SPI,   
///  
#	define	PORT_CLK		ONE_PORT
///  
#	define	PIN_CLK			2

///    
#	define	PORT_DI			ONE_PORT
///    
#	define	PIN_DI			4
#endif

///       
#define ANIMATE_DELAY_MS	10

///    
typedef enum{				// real pwm
	PWM_BRIGHT_000	= 0,	// 0.0%
	PWM_BRIGHT_002	= 1,	// 1.7%
	PWM_BRIGHT_003	= 2, 	// 2.2%
	PWM_BRIGHT_005	= 3, 	// 5.0%
	PWM_BRIGHT_007	= 4, 	// 6.7%
	PWM_BRIGHT_008	= 5, 	// 8.3%
	PWM_BRIGHT_012	= 6, 	// 11.7%
	PWM_BRIGHT_015	= 7, 	// 15%
	PWM_BRIGHT_018	= 8, 	// 18%
	PWM_BRIGHT_023	= 9, 	// 23%
	PWM_BRIGHT_030	= 10,	// 30%
	PWM_BRIGHT_037	= 11,	// 37%
	PWM_BRIGHT_047	= 12,	// 47%
	PWM_BRIGHT_060	= 13,	// 60%
	PWM_BRIGHT_080	= 14,	// 80%
	PWM_PRIGHT_100	= 15 	// 100%
} pwm_brightness_t;

///   
#define MIN_BRIGHTNESS		PWM_BRIGHT_002
///   
#define MAX_BRIGHTNESS		PWM_PRIGHT_100
///   
#define DEFAULT_BRIGHTNESS	PWM_BRIGHT_047

///     
typedef enum{				// real peak current
	PEAK_04_MA		= 2,	//  4.0 mA
	PEAK_06_MA		= 1,	//  6.4 mA
	PEAK_09_MA		= 0,	//  9.3 mA
	PEAK_13_MA		= 3		// 12.8 mA
} peak_current_t;

///    
#define DEFAULT_PEAK_CURRENT	PEAK_09_MA

///  0-  
typedef	union{
	struct{
		pwm_brightness_t
				brightness		: 4;	/// 
		peak_current_t
				peak_current	: 2;	///  
		uint8_t	sleep_mode		: 1;	///  
		uint8_t reg				: 1;	///   (  0)
	};
	uint8_t		byte;					///   ,  1 
} ctrl_reg0_t;

///  1-  
typedef	union{
	struct{
		uint8_t	data_out_ctrl	: 1;	///     DOUT
		uint8_t	edo_prescaler	: 1;	///    
		uint8_t	reserved		: 5;	///   !
		uint8_t	reg				: 1;	///   (  1)
	};
	uint8_t		byte;					///   ,  1 
} ctrl_reg1_t;

//       
#if (CHIP_RAW_SIZE > UINT8_MAX)
#define buf_size_t	uint16_t
#else
#define buf_size_t	uint8_t
#endif

///   
extern char hcms_screen[SCR_SIZE];

///  
void hcms_clrscr(void);

#if defined(USE_CURSOR)
///     
#define NO_CURSOR	(SCR_SIZE+1)
/**   
 *   ,   #NO_CURSOR
 * @param pos    ,   0
 */
void hcms_cursor_pos(uint8_t pos);
#endif

/**    
 *     ,    .  
 * ,     .
 * @param s  
 */
void hcms_puts(char *s);

/**    flash  
 * ,    , ,   ,
 *      
 * @param s   flash
 */
void hcms_puts_P(const char *s);

/**  
 *      
 */
void hcms_update_scr(void);

/**   
 *
 * @param br  
 */
void hcms_bright(pwm_brightness_t br);

/**    
 *
 * @param pc   
 */
void hcms_peak_current(peak_current_t pc);

/** / 
 *       sleep.
 * @param on  - ,   -  
 */
void hcms_on(uint8_t on);

/**       
 *          
 *  .
 * @param s  
 * @param up   (0   - ,  )
 */
void hcms_rollower_puts(char *s, int8_t delta);

/**        
 *    #hcms_rollover_puts     
 * /   ,  : 
 *         
 * @param s  
 * @param up  (0   - ,  - )
 */
void hcms_smooth_rollower_puts(char *s, int8_t up);

//    
/**    
 *      .
 * @param buf   
 * @param sz   
 */
void hcms_raw_pixels(uint8_t *buf, buf_size_t sz);

#endif /* HCMS_297X_H_ */
